@@ -1,4 +1,4 @@
-use bin_layout::{DataType, Cursor, ErrorKind, Record, Bytes};
+use bin_layout::*;
 use Subject::*;
 
 #[derive(PartialEq, Debug, Clone)]
@@ -9,45 +9,48 @@ enum Subject<'a> {
     Other(u16, Record<u8, &'a str>),
 }
 
-impl<'de> DataType<'de> for Subject<'de> {
+impl<'a> Encoder for Subject<'a> {
     fn size_hint(&self) -> usize {
         match self {
-            Other(a, r) =>  a.size_hint() + r.size_hint(),
+            Other(a, r) => a.size_hint() + r.size_hint(),
             _ => 2,
         }
     }
-    fn serialize(self, view: &mut Cursor<impl Bytes>) {
+    fn encoder(self, view: &mut Cursor<impl Bytes>) {
         let code: u16 = match self {
             Math => 302,
             Physics => 317,
             Chemistry => 345,
             Other(id, name) => {
-                id.serialize(view);
-                return Record::serialize(name, view);
+                id.encoder(view);
+                return Record::encoder(name, view);
             }
         };
-        code.serialize(view)
+        code.encoder(view)
     }
-
-    fn deserialize(view: &mut Cursor<&'de [u8]>) -> Result<Self, ErrorKind> {
-        let name = match u16::deserialize(view)? {
+}
+impl<'de, E: Error> Decoder<'de, E> for Subject<'de> {
+    fn decoder(view: &mut Cursor<&'de [u8]>) -> Result<Self, E> {
+        let name = match u16::decoder(view)? {
             302 => Math,
             317 => Physics,
             345 => Chemistry,
-            id => return Ok(Other(id, Record::deserialize(view)?)),
+            id => return Ok(Other(id, Record::decoder(view)?)),
         };
         Ok(name)
     }
 }
 
-#[derive(PartialEq, DataType, Debug, Clone)]
+#[derive(PartialEq, Encoder, Decoder, Debug, Clone)]
 struct Student<'a> {
     age: u8,
     name: &'a str,
     gender: bool,
     roll: u8,
 }
-#[derive(DataType, Debug, PartialEq, Clone)]
+
+
+#[derive(Encoder, Decoder, Debug, PartialEq, Clone)]
 struct Class<'a> {
     name: &'a str,
     subjects: [Subject<'a>; 4],
@@ -55,13 +58,23 @@ struct Class<'a> {
 }
 
 #[test]
-fn basic() {
+fn basic() -> Result<(), ()> {
     let old_class = Class {
         name: "Mango",
         subjects: [Physics, Chemistry, Other(321, "Engish II".into()), Math],
         students: vec![
-            Student { age: 21, name: "John", gender: true, roll: 73 },
-            Student { age: 20, name: "Jui", gender: false, roll: 36 },
+            Student {
+                age: 21,
+                name: "John",
+                gender: true,
+                roll: 73,
+            },
+            Student {
+                age: 20,
+                name: "Jui",
+                gender: false,
+                roll: 36,
+            },
         ]
         .into(),
     };
@@ -69,10 +82,11 @@ fn basic() {
     // Note: Size hint for `&str` is `Lencoder::SIZE + bytes.len()`
     // "Mango" has 5 chars + Lencoder::SIZE (L2) = 7
     assert!(old_class.size_hint() >= 43);
-    
+
     let bytes = old_class.clone().encode();
     assert_eq!(bytes.len(), 40);
 
-    let new_class = Class::decode(&bytes).unwrap();
+    let new_class = Class::decode(&bytes)?;
     assert_eq!(old_class, new_class);
-}
\ No newline at end of file
+    Ok(())
+}
